@@ -15,7 +15,8 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Authentication backend that denies all requests"""
+"""Authentication backend that denies all requests."""
+
 from __future__ import annotations
 
 from functools import wraps
@@ -27,14 +28,14 @@ CLIENT_AUTH: tuple[str, str] | Any | None = None
 
 
 def init_app(_):
-    """Initializes authentication"""
+    """Initialize authentication."""
 
 
 T = TypeVar("T", bound=Callable)
 
 
 def requires_authentication(function: T):
-    """Decorator for functions that require authentication"""
+    """Decorate functions that require authentication."""
 
     @wraps(function)
     def decorated(*args, **kwargs):
