@@ -15,12 +15,15 @@
 # specific language governing permissions and limitations
 # under the License.
 """Session authentication backend."""
+
 from __future__ import annotations
 
 from functools import wraps
 from typing import Any, Callable, TypeVar, cast
 
-from flask import Response, g
+from flask import Response
+
+from airflow.www.extensions.init_auth_manager import get_auth_manager
 
 CLIENT_AUTH: tuple[str, str] | Any | None = None
 
@@ -37,7 +40,7 @@ def requires_authentication(function: T):
 
     @wraps(function)
     def decorated(*args, **kwargs):
-        if g.user.is_anonymous:
+        if not get_auth_manager().is_logged_in():
             return Response("Unauthorized", 401, {})
         return function(*args, **kwargs)
 
