@@ -53,7 +53,8 @@ def _create_dagruns(
     state: DagRunState,
     run_type: DagRunType,
 ) -> Iterable[DagRun]:
-    """Infers from data intervals which DAG runs need to be created and does so.
+    """
+    Infers from data intervals which DAG runs need to be created and does so.
 
     :param dag: The DAG to create runs for.
     :param infos: List of logical dates and data intervals to evaluate.
@@ -162,10 +163,6 @@ def set_state(
             qry_sub_dag = all_subdag_tasks_query(sub_dag_run_ids, session, state, confirmed_dates)
             tis_altered += session.scalars(qry_sub_dag.with_for_update()).all()
         for task_instance in tis_altered:
-            # The try_number was decremented when setting to up_for_reschedule and deferred.
-            # Increment it back when changing the state again
-            if task_instance.state in (TaskInstanceState.DEFERRED, TaskInstanceState.UP_FOR_RESCHEDULE):
-                task_instance._try_number += 1
             task_instance.set_state(state, session=session)
         session.flush()
     else:
@@ -219,7 +216,8 @@ def _iter_subdag_run_ids(
     commit: bool,
     confirmed_infos: Iterable[_DagRunInfo],
 ) -> Iterator[str]:
-    """Go through subdag operators and create dag runs.
+    """
+    Go through subdag operators and create dag runs.
 
     We only work within the scope of the subdag. A subdag does not propagate to
     its parent DAG, but parent propagates to subdags.
@@ -258,7 +256,8 @@ def verify_dagruns(
     session: SASession,
     current_task: Operator,
 ):
-    """Verify integrity of dag_runs.
+    """
+    Verify integrity of dag_runs.
 
     :param dag_runs: dag runs to verify
     :param commit: whether dag runs state should be updated
