@@ -22,7 +22,7 @@ from typing import TYPE_CHECKING, Any, Iterable, Iterator, Mapping, Sequence
 
 import attr
 import pendulum
-from sqlalchemy import select, tuple_, update
+from sqlalchemy import case, or_, select, tuple_, update
 from sqlalchemy.exc import OperationalError
 from sqlalchemy.orm.session import make_transient
 from tabulate import tabulate
@@ -35,6 +35,7 @@ from airflow.exceptions import (
     NoAvailablePoolSlot,
     PoolNotFound,
     TaskConcurrencyLimitReached,
+    UnknownExecutorException,
 )
 from airflow.executors.executor_loader import ExecutorLoader
 from airflow.jobs.base_job_runner import BaseJobRunner
@@ -46,7 +47,7 @@ from airflow.ti_deps.dep_context import DepContext
 from airflow.ti_deps.dependencies_deps import BACKFILL_QUEUED_DEPS
 from airflow.timetables.base import DagRunInfo
 from airflow.utils import helpers, timezone
-from airflow.utils.configuration import conf as airflow_conf, tmp_configuration_copy
+from airflow.utils.configuration import tmp_configuration_copy
 from airflow.utils.log.logging_mixin import LoggingMixin
 from airflow.utils.session import NEW_SESSION, provide_session
 from airflow.utils.state import DagRunState, State, TaskInstanceState
@@ -245,7 +246,16 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
             session.execute(
                 update(TI)
                 .where(filter_for_tis)
-                .values(state=TaskInstanceState.SCHEDULED)
+                .values(
+                    state=TaskInstanceState.SCHEDULED,
+                    try_number=case(
+                        (
+                            or_(TI.state.is_(None), TI.state != TaskInstanceState.UP_FOR_RESCHEDULE),
+                            TI.try_number + 1,
+                        ),
+                        else_=TI.try_number,
+                    ),
+                )
                 .execution_options(synchronize_session=False)
             )
             session.flush()
@@ -253,6 +263,7 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
     def _manage_executor_state(
         self,
         running: Mapping[TaskInstanceKey, TaskInstance],
+        executor: BaseExecutor,
         session: Session,
     ) -> Iterator[tuple[AbstractOperator, str, Sequence[TaskInstance], int]]:
         """
@@ -263,7 +274,6 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
         :param running: dict of key, task to verify
         :return: An iterable of expanded TaskInstance per MappedTask
         """
-        executor = self.job.executor
         # list of tuples (dag_id, task_id, execution_date, map_index) of running tasks in executor
         buffered_events = list(executor.get_event_buffer().items())
         running_tis_ids = [
@@ -428,6 +438,8 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
         try:
             for ti in dag_run.get_task_instances(session=session):
                 if ti in schedulable_tis:
+                    if ti.state != TaskInstanceState.UP_FOR_RESCHEDULE:
+                        ti.try_number += 1
                     ti.set_state(TaskInstanceState.SCHEDULED)
                 if ti.state != TaskInstanceState.REMOVED:
                     tasks_to_run[ti.key] = ti
@@ -457,7 +469,6 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
     def _process_backfill_task_instances(
         self,
         ti_status: _DagRunTaskStatus,
-        executor: BaseExecutor,
         pickle_id: int | None,
         start_date: datetime.datetime | None = None,
         *,
@@ -478,10 +489,8 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
         """
         executed_run_dates = []
 
-        is_unit_test = airflow_conf.getboolean("core", "unit_test_mode")
-
         while (ti_status.to_run or ti_status.running) and not ti_status.deadlocked:
-            self.log.debug("*** Clearing out not_ready list ***")
+            self.log.debug("Clearing out not_ready list")
             ti_status.not_ready.clear()
 
             # we need to execute the tasks bottom to top
@@ -520,6 +529,7 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
                         if key in ti_status.running:
                             ti_status.running.pop(key)
                         # Reset the failed task in backfill to scheduled state
+                        ti.try_number += 1
                         ti.set_state(TaskInstanceState.SCHEDULED, session=session)
                         if ti.dag_run not in ti_status.active_runs:
                             ti_status.active_runs.add(ti.dag_run)
@@ -547,6 +557,7 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
                     flag_upstream_failed=True,
                 )
 
+                executor = ExecutorLoader.load_executor(str(ti.executor) if ti.executor else None)
                 # Is the task runnable? -- then run it
                 # the dependency checker can change states of tis
                 if ti.are_dependencies_met(
@@ -557,6 +568,14 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
                     else:
                         self.log.debug("Sending %s to executor", ti)
                         # Skip scheduled state, we are executing immediately
+                        if ti.state in (TaskInstanceState.UP_FOR_RETRY, None):
+                            # i am not sure why this is necessary.
+                            # seemingly a quirk of backfill runner.
+                            # it should be handled elsewhere i think.
+                            # seems the leaf tasks are set SCHEDULED but others not.
+                            # but i am not going to look too closely since we need
+                            # to nuke the current backfill approach anyway.
+                            ti.try_number += 1
                         ti.state = TaskInstanceState.QUEUED
                         ti.queued_by_job_id = self.job.id
                         ti.queued_dttm = timezone.utcnow()
@@ -700,10 +719,13 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
                 self.log.debug(e)
 
             perform_heartbeat(
-                job=self.job, heartbeat_callback=self.heartbeat_callback, only_if_necessary=is_unit_test
+                job=self.job,
+                heartbeat_callback=self.heartbeat_callback,
+                only_if_necessary=True,
             )
             # execute the tasks in the queue
-            executor.heartbeat()
+            for executor in self.job.executors:
+                executor.heartbeat()
 
             # If the set of tasks that aren't ready ever equals the set of
             # tasks to run and there are no running tasks then the backfill
@@ -713,24 +735,26 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
                 ti_status.deadlocked.update(ti_status.to_run.values())
                 ti_status.to_run.clear()
 
-            # check executor state -- and expand any mapped TIs
-            for node, run_id, new_mapped_tis, max_map_index in self._manage_executor_state(
-                ti_status.running, session
-            ):
+            for executor in self.job.executors:
+                # check executor state -- and expand any mapped TIs
+                for node, run_id, new_mapped_tis, max_map_index in self._manage_executor_state(
+                    ti_status.running, executor, session
+                ):
 
-                def to_keep(key: TaskInstanceKey) -> bool:
-                    if key.dag_id != node.dag_id or key.task_id != node.task_id or key.run_id != run_id:
-                        # For another Dag/Task/Run -- don't remove
-                        return True
-                    return 0 <= key.map_index <= max_map_index
+                    def to_keep(key: TaskInstanceKey) -> bool:
+                        if key.dag_id != node.dag_id or key.task_id != node.task_id or key.run_id != run_id:
+                            # For another Dag/Task/Run -- don't remove
+                            return True
+                        return 0 <= key.map_index <= max_map_index
 
-                # remove the old unmapped TIs for node -- they have been replaced with the mapped TIs
-                ti_status.to_run = {key: ti for (key, ti) in ti_status.to_run.items() if to_keep(key)}
+                    # remove the old unmapped TIs for node -- they have been replaced with the mapped TIs
+                    ti_status.to_run = {key: ti for (key, ti) in ti_status.to_run.items() if to_keep(key)}
 
-                ti_status.to_run.update({ti.key: ti for ti in new_mapped_tis})
+                    ti_status.to_run.update({ti.key: ti for ti in new_mapped_tis})
 
-                for new_ti in new_mapped_tis:
-                    new_ti.set_state(TaskInstanceState.SCHEDULED, session=session)
+                    for new_ti in new_mapped_tis:
+                        new_ti.try_number += 1
+                        new_ti.set_state(TaskInstanceState.SCHEDULED, session=session)
 
             # Set state to failed for running TIs that are set up for retry if disable-retry flag is set
             for ti in ti_status.running.values():
@@ -752,6 +776,7 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
 
             self._log_progress(ti_status)
             session.commit()
+            time.sleep(1)
 
         # return updated status
         return executed_run_dates
@@ -822,7 +847,6 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
         self,
         dagrun_infos: Iterable[DagRunInfo],
         ti_status: _DagRunTaskStatus,
-        executor: BaseExecutor,
         pickle_id: int | None,
         start_date: datetime.datetime | None,
         session: Session = NEW_SESSION,
@@ -834,7 +858,6 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
 
         :param dagrun_infos: Schedule information for dag runs
         :param ti_status: internal BackfillJobRunner status structure to tis track progress
-        :param executor: the executor to use, it must be previously started
         :param pickle_id: numeric id of the pickled dag, None if not pickled
         :param start_date: backfill start date
         :param session: the current session object
@@ -847,9 +870,25 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
                     ti_status.active_runs.add(dag_run)
                     ti_status.to_run.update(tis_map or {})
 
+        tis_missing_executor = []
+        for ti in ti_status.to_run.values():
+            if ti.executor:
+                try:
+                    ExecutorLoader.lookup_executor_name_by_str(ti.executor)
+                except UnknownExecutorException:
+                    tis_missing_executor.append(ti)
+
+        if tis_missing_executor:
+            raise UnknownExecutorException(
+                "The following task instances are configured to use an executor that is not present. "
+                "Review the core.executors Airflow configuration to add it or clear the task instance to "
+                "clear the executor configuration for this task.\n"
+                + "\n".join(
+                    [f"  {ti.task_id}: {ti.run_id} (executor: {ti.executor})" for ti in tis_missing_executor]
+                )
+            )
         processed_dag_run_dates = self._process_backfill_task_instances(
             ti_status=ti_status,
-            executor=executor,
             pickle_id=pickle_id,
             start_date=start_date,
             session=session,
@@ -934,21 +973,22 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
                 "combination. Please adjust backfill dates or wait for this DagRun to finish.",
             )
             return
-        # picklin'
         pickle_id = None
 
-        executor_class, _ = ExecutorLoader.import_default_executor_cls()
+        _support_pickling = []
 
-        if not self.donot_pickle and executor_class.supports_pickling:
+        for executor in self.job.executors:
+            _support_pickling.append(executor.supports_pickling)
+
+            executor.job_id = self.job.id
+            executor.start()
+
+        if not self.donot_pickle and all(_support_pickling):
             pickle = DagPickle(self.dag)
             session.add(pickle)
             session.commit()
             pickle_id = pickle.id
 
-        executor = self.job.executor
-        executor.job_id = self.job.id
-        executor.start()
-
         ti_status.total_runs = len(dagrun_infos)  # total dag runs in backfill
 
         try:
@@ -962,7 +1002,6 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
                 self._execute_dagruns(
                     dagrun_infos=dagrun_infos_to_process,
                     ti_status=ti_status,
-                    executor=executor,
                     pickle_id=pickle_id,
                     start_date=start_date,
                     session=session,
@@ -995,7 +1034,8 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
             raise
         finally:
             session.commit()
-            executor.end()
+            for executor in self.job.executors:
+                executor.end()
 
         self.log.info("Backfill done for DAG %s. Exiting.", self.dag)
 
@@ -1017,9 +1057,12 @@ class BackfillJobRunner(BaseJobRunner, LoggingMixin):
         :param filter_by_dag_run: the dag_run we want to process, None if all
         :return: the number of TIs reset
         """
-        queued_tis = self.job.executor.queued_tasks
-        # also consider running as the state might not have changed in the db yet
-        running_tis = self.job.executor.running
+        queued_tis = []
+        running_tis = []
+        for executor in self.job.executors:
+            queued_tis.append(executor.queued_tasks)
+            # also consider running as the state might not have changed in the db yet
+            running_tis.append(executor.running)
 
         # Can't use an update here since it doesn't support joins.
         resettable_states = [TaskInstanceState.SCHEDULED, TaskInstanceState.QUEUED]
