@@ -15,21 +15,23 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Maintain history for compatibility with earlier migrations
+"""
+Maintain history for compatibility with earlier migrations.
 
 Revision ID: 13eb55f81627
 Revises: 1507a7289a2f
 Create Date: 2015-08-23 05:12:49.732174
 
 """
+
 from __future__ import annotations
 
 # revision identifiers, used by Alembic.
-revision = '13eb55f81627'
-down_revision = '1507a7289a2f'
+revision = "13eb55f81627"
+down_revision = "1507a7289a2f"
 branch_labels = None
 depends_on = None
-airflow_version = '1.5.0'
+airflow_version = "1.5.0"
 
 
 def upgrade():
