@@ -21,14 +21,16 @@ import os
 import shutil
 import warnings
 from functools import cached_property
-from typing import TYPE_CHECKING, Container, Sequence
+from typing import TYPE_CHECKING, Any, Callable, Container, Sequence, cast
 
 from airflow.exceptions import AirflowException, AirflowSkipException
 from airflow.hooks.subprocess import SubprocessHook
 from airflow.models.baseoperator import BaseOperator
 from airflow.utils.operator_helpers import context_to_airflow_vars
+from airflow.utils.types import ArgNotSet
 
 if TYPE_CHECKING:
+    from airflow.models.taskinstance import TaskInstance
     from airflow.utils.context import Context
 
 
@@ -44,7 +46,7 @@ class BashOperator(BaseOperator):
     will also be pushed to an XCom when the bash command completes
 
     :param bash_command: The command, set of commands or reference to a
-        bash script (must be '.sh') to be executed. (templated)
+        Bash script (must be '.sh' or '.bash') to be executed. (templated)
     :param env: If env is not None, it must be a dict that defines the
         environment variables for the new process; these are used instead
         of inheriting the current process environment, which is the default
@@ -53,14 +55,18 @@ class BashOperator(BaseOperator):
         and does not inherit the current process environment. If True, inherits the environment variables
         from current passes and then environment variable passed by the user will either update the existing
         inherited environment variables or the new variables gets appended to it
-    :param output_encoding: Output encoding of bash command
+    :param output_encoding: Output encoding of Bash command
     :param skip_on_exit_code: If task exits with this exit code, leave the task
         in ``skipped`` state (default: 99). If set to ``None``, any non-zero
         exit code will be treated as a failure.
-    :param cwd: Working directory to execute the command in.
+    :param cwd: Working directory to execute the command in (templated).
         If None (default), the command is run in a temporary directory.
+        To use current DAG folder as the working directory,
+        you might set template ``{{ dag_run.dag.folder }}``.
+    :param output_processor: Function to further process the output of the bash script
+        (default is lambda output: output).
 
-    Airflow will evaluate the exit code of the bash command. In general, a non-zero exit code will result in
+    Airflow will evaluate the exit code of the Bash command. In general, a non-zero exit code will result in
     task failure and zero will result in task success.
     Exit code ``99`` (or another set in ``skip_on_exit_code``)
     will throw an :class:`airflow.exceptions.AirflowSkipException`, which will leave the task in ``skipped``
@@ -85,7 +91,6 @@ class BashOperator(BaseOperator):
         code.  This can be an issue if the non-zero exit arises from a sub-command.  The easiest way of
         addressing this is to prefix the command with ``set -e;``
 
-        Example:
         .. code-block:: python
 
             bash_command = "set -e; python3 script.py '{{ next_execution_date }}'"
@@ -127,26 +132,27 @@ class BashOperator(BaseOperator):
             env={"message": '{{ dag_run.conf["message"] if dag_run else "" }}'},
         )
 
+    .. versionadded:: 2.10.0
+       The `output_processor` parameter.
+
     """
 
-    template_fields: Sequence[str] = ("bash_command", "env")
+    template_fields: Sequence[str] = ("bash_command", "env", "cwd")
     template_fields_renderers = {"bash_command": "bash", "env": "json"}
-    template_ext: Sequence[str] = (
-        ".sh",
-        ".bash",
-    )
+    template_ext: Sequence[str] = (".sh", ".bash")
     ui_color = "#f0ede4"
 
     def __init__(
         self,
         *,
-        bash_command: str,
+        bash_command: str | ArgNotSet,
         env: dict[str, str] | None = None,
         append_env: bool = False,
         output_encoding: str = "utf-8",
         skip_exit_code: int | None = None,
         skip_on_exit_code: int | Container[int] | None = 99,
         cwd: str | None = None,
+        output_processor: Callable[[str], Any] = lambda result: result,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -167,12 +173,33 @@ class BashOperator(BaseOperator):
         )
         self.cwd = cwd
         self.append_env = append_env
+        self.output_processor = output_processor
+
+        # When using the @task.bash decorator, the Bash command is not known until the underlying Python
+        # callable is executed and therefore set to NOTSET initially. This flag is useful during execution to
+        # determine whether the bash_command value needs to re-rendered.
+        self._init_bash_command_not_set = isinstance(self.bash_command, ArgNotSet)
 
     @cached_property
     def subprocess_hook(self):
         """Returns hook for running the bash command."""
         return SubprocessHook()
 
+    @staticmethod
+    def refresh_bash_command(ti: TaskInstance) -> None:
+        """
+        Rewrite the underlying rendered bash_command value for a task instance in the metadatabase.
+
+        TaskInstance.get_rendered_template_fields() cannot be used because this will retrieve the
+        RenderedTaskInstanceFields from the metadatabase which doesn't have the runtime-evaluated bash_command
+        value.
+
+        :meta private:
+        """
+        from airflow.models.renderedtifields import RenderedTaskInstanceFields
+
+        RenderedTaskInstanceFields._update_runtime_evaluated_template_fields(ti)
+
     def get_env(self, context):
         """Build the set of environment variables to be exposed for the bash command."""
         system_env = os.environ.copy()
@@ -200,6 +227,16 @@ class BashOperator(BaseOperator):
             if not os.path.isdir(self.cwd):
                 raise AirflowException(f"The cwd {self.cwd} must be a directory")
         env = self.get_env(context)
+
+        # Because the bash_command value is evaluated at runtime using the @task.bash decorator, the
+        # RenderedTaskInstanceField data needs to be rewritten and the bash_command value re-rendered -- the
+        # latter because the returned command from the decorated callable could contain a Jinja expression.
+        # Both will ensure the correct Bash command is executed and that the Rendered Template view in the UI
+        # displays the executed command (otherwise it will display as an ArgNotSet type).
+        if self._init_bash_command_not_set:
+            ti = cast("TaskInstance", context["ti"])
+            self.refresh_bash_command(ti)
+
         result = self.subprocess_hook.run_command(
             command=[bash_path, "-c", self.bash_command],
             env=env,
@@ -212,7 +249,8 @@ class BashOperator(BaseOperator):
             raise AirflowException(
                 f"Bash command failed. The command returned a non-zero exit code {result.exit_code}."
             )
-        return result.output
+
+        return self.output_processor(result.output)
 
     def on_kill(self) -> None:
         self.subprocess_hook.send_sigterm()
