@@ -64,7 +64,6 @@ from airflow.security.permissions import (
     RESOURCE_XCOM,
 )
 from airflow.utils.log.logging_mixin import LoggingMixin
-from airflow.utils.session import NEW_SESSION, provide_session
 from airflow.www.extensions.init_auth_manager import get_auth_manager
 from airflow.www.utils import CustomSQLAInterface
 
@@ -77,13 +76,12 @@ EXISTING_ROLES = {
 }
 
 if TYPE_CHECKING:
-    from sqlalchemy.orm import Session
-
     from airflow.auth.managers.models.base_user import BaseUser
 
 
 class AirflowSecurityManagerV2(LoggingMixin):
-    """Custom security manager, which introduces a permission model adapted to Airflow.
+    """
+    Custom security manager, which introduces a permission model adapted to Airflow.
 
     It's named V2 to differentiate it from the obsolete airflow.www.security.AirflowSecurityManager.
     """
@@ -142,7 +140,8 @@ class AirflowSecurityManagerV2(LoggingMixin):
         return is_authorized_method(action_name, resource_pk, user)
 
     def create_admin_standalone(self) -> tuple[str | None, str | None]:
-        """Perform the required steps when initializing airflow for standalone mode.
+        """
+        Perform the required steps when initializing airflow for standalone mode.
 
         If necessary, returns the username and password to be printed in the console for users to log in.
         """
@@ -167,9 +166,8 @@ class AirflowSecurityManagerV2(LoggingMixin):
             )(baseview.blueprint)
 
     @cached_property
-    @provide_session
     def _auth_manager_is_authorized_map(
-        self, session: Session = NEW_SESSION
+        self,
     ) -> dict[str, Callable[[str, str | None, BaseUser | None], bool]]:
         """
         Return the map associating a FAB resource name to the corresponding auth manager is_authorized_ API.
@@ -179,6 +177,8 @@ class AirflowSecurityManagerV2(LoggingMixin):
         auth_manager = get_auth_manager()
         methods = get_method_from_fab_action_map()
 
+        session = self.appbuilder.session
+
         def get_connection_id(resource_pk):
             if not resource_pk:
                 return None
@@ -226,19 +226,24 @@ class AirflowSecurityManagerV2(LoggingMixin):
                 return None
             variable = session.scalar(select(Variable).where(Variable.id == resource_pk).limit(1))
             if not variable:
-                raise AirflowException("Connection not found")
+                raise AirflowException("Variable not found")
             return variable.key
 
-        return {
-            RESOURCE_AUDIT_LOG: lambda action, resource_pk, user: auth_manager.is_authorized_dag(
+        def _is_authorized_view(view_):
+            return lambda action, resource_pk, user: auth_manager.is_authorized_view(
+                access_view=view_,
+                user=user,
+            )
+
+        def _is_authorized_dag(entity_=None, details_func_=None):
+            return lambda action, resource_pk, user: auth_manager.is_authorized_dag(
                 method=methods[action],
-                access_entity=DagAccessEntity.AUDIT_LOG,
+                access_entity=entity_,
+                details=DagDetails(id=details_func_(resource_pk)) if details_func_ else None,
                 user=user,
-            ),
-            RESOURCE_CLUSTER_ACTIVITY: lambda action, resource_pk, user: auth_manager.is_authorized_view(
-                access_view=AccessView.CLUSTER_ACTIVITY,
-                user=user,
-            ),
+            )
+
+        mapping = {
             RESOURCE_CONFIG: lambda action, resource_pk, user: auth_manager.is_authorized_configuration(
                 method=methods[action],
                 user=user,
@@ -248,82 +253,43 @@ class AirflowSecurityManagerV2(LoggingMixin):
                 details=ConnectionDetails(conn_id=get_connection_id(resource_pk)),
                 user=user,
             ),
-            RESOURCE_DAG: lambda action, resource_pk, user: auth_manager.is_authorized_dag(
-                method=methods[action],
-                user=user,
-            ),
-            RESOURCE_DAG_CODE: lambda action, resource_pk, user: auth_manager.is_authorized_dag(
-                method=methods[action],
-                access_entity=DagAccessEntity.CODE,
-                user=user,
-            ),
-            RESOURCE_DAG_DEPENDENCIES: lambda action, resource_pk, user: auth_manager.is_authorized_dag(
-                method=methods[action],
-                access_entity=DagAccessEntity.DEPENDENCIES,
-                user=user,
-            ),
-            RESOURCE_DAG_RUN: lambda action, resource_pk, user: auth_manager.is_authorized_dag(
-                method=methods[action],
-                access_entity=DagAccessEntity.RUN,
-                details=DagDetails(id=get_dag_id_from_dagrun_id(resource_pk)),
-                user=user,
-            ),
             RESOURCE_DATASET: lambda action, resource_pk, user: auth_manager.is_authorized_dataset(
                 method=methods[action],
                 user=user,
             ),
-            RESOURCE_DOCS: lambda action, resource_pk, user: auth_manager.is_authorized_view(
-                access_view=AccessView.DOCS,
-                user=user,
-            ),
-            RESOURCE_PLUGIN: lambda action, resource_pk, user: auth_manager.is_authorized_view(
-                access_view=AccessView.PLUGINS,
-                user=user,
-            ),
-            RESOURCE_JOB: lambda action, resource_pk, user: auth_manager.is_authorized_view(
-                access_view=AccessView.JOBS,
-                user=user,
-            ),
             RESOURCE_POOL: lambda action, resource_pk, user: auth_manager.is_authorized_pool(
                 method=methods[action],
                 details=PoolDetails(name=get_pool_name(resource_pk)),
                 user=user,
             ),
-            RESOURCE_PROVIDER: lambda action, resource_pk, user: auth_manager.is_authorized_view(
-                access_view=AccessView.PROVIDERS,
-                user=user,
-            ),
-            RESOURCE_SLA_MISS: lambda action, resource_pk, user: auth_manager.is_authorized_dag(
-                method=methods[action],
-                access_entity=DagAccessEntity.SLA_MISS,
-                user=user,
-            ),
-            RESOURCE_TASK_INSTANCE: lambda action, resource_pk, user: auth_manager.is_authorized_dag(
-                method=methods[action],
-                access_entity=DagAccessEntity.TASK_INSTANCE,
-                details=DagDetails(id=get_dag_id_from_task_instance(resource_pk)),
-                user=user,
-            ),
-            RESOURCE_TASK_RESCHEDULE: lambda action, resource_pk, user: auth_manager.is_authorized_dag(
-                method=methods[action],
-                access_entity=DagAccessEntity.TASK_RESCHEDULE,
-                user=user,
-            ),
-            RESOURCE_TRIGGER: lambda action, resource_pk, user: auth_manager.is_authorized_view(
-                access_view=AccessView.TRIGGERS,
-                user=user,
-            ),
             RESOURCE_VARIABLE: lambda action, resource_pk, user: auth_manager.is_authorized_variable(
                 method=methods[action],
                 details=VariableDetails(key=get_variable_key(resource_pk)),
                 user=user,
             ),
-            RESOURCE_XCOM: lambda action, resource_pk, user: auth_manager.is_authorized_dag(
-                method=methods[action],
-                access_entity=DagAccessEntity.XCOM,
-                user=user,
-            ),
         }
+        for resource, entity, details_func in [
+            (RESOURCE_DAG, None, None),
+            (RESOURCE_AUDIT_LOG, DagAccessEntity.AUDIT_LOG, None),
+            (RESOURCE_DAG_CODE, DagAccessEntity.CODE, None),
+            (RESOURCE_DAG_DEPENDENCIES, DagAccessEntity.DEPENDENCIES, None),
+            (RESOURCE_SLA_MISS, DagAccessEntity.SLA_MISS, None),
+            (RESOURCE_TASK_RESCHEDULE, DagAccessEntity.TASK_RESCHEDULE, None),
+            (RESOURCE_XCOM, DagAccessEntity.XCOM, None),
+            (RESOURCE_DAG_RUN, DagAccessEntity.RUN, get_dag_id_from_dagrun_id),
+            (RESOURCE_TASK_INSTANCE, DagAccessEntity.TASK_INSTANCE, get_dag_id_from_task_instance),
+        ]:
+            mapping[resource] = _is_authorized_dag(entity, details_func)
+        for resource, view in [
+            (RESOURCE_CLUSTER_ACTIVITY, AccessView.CLUSTER_ACTIVITY),
+            (RESOURCE_DOCS, AccessView.DOCS),
+            (RESOURCE_PLUGIN, AccessView.PLUGINS),
+            (RESOURCE_JOB, AccessView.JOBS),
+            (RESOURCE_PROVIDER, AccessView.PROVIDERS),
+            (RESOURCE_TRIGGER, AccessView.TRIGGERS),
+        ]:
+            mapping[resource] = _is_authorized_view(view)
+        return mapping
 
     def _get_auth_manager_is_authorized_method(self, fab_resource_name: str) -> Callable:
         is_authorized_method = self._auth_manager_is_authorized_map.get(fab_resource_name)
@@ -337,7 +303,7 @@ class AirflowSecurityManagerV2(LoggingMixin):
             # The user is trying to access a page specific to the auth manager
             # (e.g. the user list view in FabAuthManager) or a page defined in a plugin
             return lambda action, resource_pk, user: get_auth_manager().is_authorized_custom_view(
-                method=get_method_from_fab_action_map()[action],
+                method=get_method_from_fab_action_map().get(action, action),
                 resource_name=fab_resource_name,
                 user=user,
             )
