@@ -33,6 +33,7 @@ import useGraphData from "./useGraphData";
 import useGridData from "./useGridData";
 import useMappedInstances from "./useMappedInstances";
 import useDatasets from "./useDatasets";
+import useDatasetsSummary from "./useDatasetsSummary";
 import useDataset from "./useDataset";
 import useDatasetDependencies from "./useDatasetDependencies";
 import useDatasetEvents from "./useDatasetEvents";
@@ -40,6 +41,29 @@ import useSetDagRunNote from "./useSetDagRunNote";
 import useSetTaskInstanceNote from "./useSetTaskInstanceNote";
 import useUpstreamDatasetEvents from "./useUpstreamDatasetEvents";
 import useTaskInstance from "./useTaskInstance";
+import useTaskFailedDependency from "./useTaskFailedDependency";
+import useDag from "./useDag";
+import useDagCode from "./useDagCode";
+import useDagDetails from "./useDagDetails";
+import useHealth from "./useHealth";
+import usePools from "./usePools";
+import useDags from "./useDags";
+import useDagRuns from "./useDagRuns";
+import useHistoricalMetricsData from "./useHistoricalMetricsData";
+import { useTaskXcomEntry, useTaskXcomCollection } from "./useTaskXcom";
+import useEventLogs from "./useEventLogs";
+import useCalendarData from "./useCalendarData";
+import useCreateDatasetEvent from "./useCreateDatasetEvent";
+import useRenderedK8s from "./useRenderedK8s";
+import useTaskDetail from "./useTaskDetail";
+import useTIHistory from "./useTIHistory";
+
+axios.interceptors.request.use((config) => {
+  config.paramsSerializer = {
+    indexes: null,
+  };
+  return config;
+});
 
 axios.interceptors.response.use((res: AxiosResponse) =>
   res.data ? camelcaseKeys(res.data, { deep: true }) : res
@@ -50,22 +74,40 @@ axios.defaults.headers.common.Accept = "application/json";
 export {
   useClearRun,
   useClearTask,
-  useMarkTaskDryRun,
+  useDag,
+  useDagCode,
+  useDagDetails,
+  useDagRuns,
+  useDags,
   useDataset,
+  useDatasets,
   useDatasetDependencies,
   useDatasetEvents,
-  useDatasets,
+  useDatasetsSummary,
   useExtraLinks,
   useGraphData,
   useGridData,
+  useHealth,
   useMappedInstances,
   useMarkFailedRun,
   useMarkFailedTask,
   useMarkSuccessRun,
   useMarkSuccessTask,
+  useMarkTaskDryRun,
+  usePools,
   useQueueRun,
   useSetDagRunNote,
   useSetTaskInstanceNote,
   useTaskInstance,
   useUpstreamDatasetEvents,
+  useHistoricalMetricsData,
+  useTaskXcomEntry,
+  useTaskXcomCollection,
+  useTaskFailedDependency,
+  useEventLogs,
+  useCalendarData,
+  useCreateDatasetEvent,
+  useRenderedK8s,
+  useTaskDetail,
+  useTIHistory,
 };
